using System;
using System.IO;
using System.Windows.Forms;

namespace TexturePicker
{
	/// <summary>
	/// Summary description for App.
	/// </summary>
	public class App
	{
		/************************************************
		 * Constants
		 ***********************************************/
		static private readonly string[] imageMasks = new string[] {"*.png","*.jpg","*.bmp"};

		/************************************************
		 * Member Variables
		 ***********************************************/
		static private ExportProgress exportProgress;
		static private string sourcePath;
		static private string targetPath;
		static private TextureCollection textures;
		
		/************************************************
		 * Constructors
		 ***********************************************/
		/// <summary>
		/// Initializes the static <see cref="App"/> instance.
		/// </summary>
		static App()
		{
			// Create vars
			exportProgress = new ExportProgress();
			textures = new TextureCollection();
		}

		/************************************************
		 * Internal Methods
		 ***********************************************/
		/// <summary>
		/// Gets the source name for a directory.
		/// </summary>
		/// <param name="directory">
		/// The directory that is the source.
		/// </param>
		/// <returns>
		/// The source name.
		/// </returns>
		static private string GetSourceName(DirectoryInfo directory)
		{
			// Start with the full directory path
			string sourceName = directory.FullName;

			// Strip out source path
			sourceName = sourceName.Replace(sourcePath, string.Empty);

			// Trim off slashes
			sourceName = sourceName.Trim('\\', '/');

			// Return new name
			return sourceName;
		}

		/// <summary>
		/// Gets the texture name for a file name.
		/// </summary>
		/// <param name="file">
		/// The file to obtain a texture name for.
		/// </param>
		/// <returns>
		/// The texture name.
		/// </returns>
		static private string GetTextureName(FileInfo file)
		{
			// Start with the file without extension
			string textureName = Path.GetFileNameWithoutExtension(file.Name);

			// Look for an underscore
			int underIndex = textureName.IndexOf('_');

			// If underscore is found, truncate at underscore
			if (underIndex > -1) textureName = textureName.Substring(0, underIndex - 1);

			// Retun new name
			return textureName;
		}

		/// <summary>
		/// Loads all texture files in a specifid sub-directory matchin a specified file mask.
		/// </summary>
		/// <param name="directory">
		/// The directory to search for files in.
		/// </param>
		/// <param name="sourceName">
		/// The source name for the directory (to avoid obtaining it every time).
		/// </param>
		/// <param name="imageMask">
		/// The mask to use when searching for images. i.e. "*.png".
		/// </param>
		static private void LoadTextures(DirectoryInfo directory, string sourceName, string imageMask)
		{
			// Get all files for the directory
			FileInfo[] files = directory.GetFiles(imageMask);

			// Loop through all matching files
			foreach (FileInfo file in files)
			{
				// Get the texture name from the file
				string textureName = GetTextureName(file);

				// See if the texutre has already been cached
				Texture texture = textures[textureName];

				// If not in cache, create and add it
				if (texture == null)
				{
					// Creat texture
					texture = new Texture(textureName);
					
					// Add texture
					textures.Add(texture);
				}

				// Now that we have a texture, try to locate the source for our current folder
				Source source = texture.Sources[sourceName];

				// If the source doesn't exist yet, create and add it
				if (source == null)
				{
					// Create source
					source = new Source(sourceName);
					
					// Add source
					texture.Sources.Add(source);

					// If there is no selected source yet, select this one
					if (texture.SelectedSource == null) texture.SelectedSource = source;
				}

				// Add the file as an associated file
				source.AssociatedFiles.Add(file);
			}
		}

		/// <summary>
		/// Loads all textures in a specified directory and sub-directories.
		/// </summary>
		/// <param name="directory">
		/// The directory to load textures from.
		/// </param>
		static private void LoadTextures(DirectoryInfo directory)
		{
			// Get the source name from the directory
			string sourceName = GetSourceName(directory);

			// Loop through all matching files
			foreach (string imageMask in imageMasks)
			{
				LoadTextures(directory, sourceName, imageMask);
			}

			// Get sub-directories
			DirectoryInfo[] subDirectories = directory.GetDirectories();

			// Loop through sub-directories
			foreach (DirectoryInfo subDirectory in subDirectories)
			{
				LoadTextures(subDirectory);
			}
		}

		/// <summary>
		/// Loads all textures from the source path.
		/// </summary>
		static private void LoadTextures()
		{
			// Clear the list of existing textures
			textures.Clear();

			// Get the directory info for the source path
			DirectoryInfo directory = new DirectoryInfo(sourcePath);

			// Load textures
			LoadTextures(directory);
		}

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			// Enable XP themes
			Application.EnableVisualStyles();
			Application.DoEvents();

			// Try to create a new project. If not successful, drop out.
			if (NewProject())
			{
				// Run application pump on main UI
				Application.Run(new MainUI());
			}
		}

		/************************************************
		 * Internal Methods
		 ***********************************************/
		/// <summary>
		/// Deletes all files that match a specified path and mask.
		/// </summary>
		/// <param name="path">
		/// The path to the folder containing files.
		/// </param>
		/// <param name="searchPattern">
		/// The search pattern of the files to delete.
		/// </param>
		static private void DeleteFiles(string path, string searchPattern)
		{
			// Get the files to delete
			string[] delFiles = Directory.GetFiles(path, searchPattern);

			// Delete each file
			foreach (string delFile in delFiles)
			{
				File.Delete(delFile);
			}
		}

		/************************************************
		 * Methods
		 ***********************************************/
		/// <summary>
		/// Exports selected textures to the target path.
		/// </summary>
		static public void ExportTextures()
		{
			// Reset the export dialog
			exportProgress.Reset();

			// Set texture count
			exportProgress.TextureCount = textures.Count;

			// Show the export dialog
			exportProgress.Show();

			// Pause for a refresh
			Application.DoEvents();

			// Input breather
			int breather = 0;

			// Loop through all textures
			foreach (Texture texture in textures)
			{
				// Breathe?
				breather++;
				if (breather % 10 == 0) Application.DoEvents();

				// Update export status and see if we should continue
				bool canContinue = exportProgress.NextTexture(texture);

				// If not continue, bail
				if (!canContinue) break;

				// If any texture-related files exist in the target path, delete them.
				DeleteFiles(targetPath, texture.Name + "*.*");

				// Get the selected source
				Source source = texture.SelectedSource;

				// Loop through all files associated with the source
				foreach (FileInfo file in source.AssociatedFiles)
				{
					// Get new path
					string newPath = string.Format("{0}{1}{2}", targetPath, Path.DirectorySeparatorChar, file.Name);

					// Copy file to output path
					file.CopyTo(newPath, true);
				}
			}

			// Hide export
			exportProgress.Hide();

			// Done
			MessageBox.Show("Done!");
		}

		/// <summary>
		/// Creates a new 'project' by prompting the user for the source and target folders.
		/// </summary>
		/// <returns>
		/// true if a new project was created; otherwise false.
		/// </returns>
		static public bool NewProject()
		{
			// Create a SelectFolders dialog
			SelectFolders selectFolders = new SelectFolders();

			// Show the dialog and get the result
			DialogResult result = selectFolders.ShowDialog();

			// If the dialog was successful, run the main app. Otherwise, just drop out.
			if (result == DialogResult.OK)
			{
				// Store paths
				sourcePath = selectFolders.SourcePath;
				targetPath = selectFolders.TargetPath;

				// Load textures
				LoadTextures();

				// Success
				return true;
			}

			// Failure
			return false;
		}

		/************************************************
		 * Properties
		 ***********************************************/		
		/// <summary>
		/// Gets the source path of the current project.
		/// </summary>
		/// <value>
		/// The source path of the current project.
		/// </value>
		static public string SourcePath
		{
			get
			{
				return sourcePath;
			}
			set
			{
				// Ensure changing
				if (sourcePath == value) return;
  
				// Store
				sourcePath = value;
			}
		}

		/// <summary>
		/// Gets the target path of the current project.
		/// </summary>
		/// <value>
		/// The target path of the current project.
		/// </value>
		static public string TargetPath
		{
			get
			{
				return sourcePath;
			}
			set
			{
				// Ensure changing
				if (sourcePath == value) return;
  
				// Store
				sourcePath = value;
			}
		}

		/// <summary>
		/// Gets a list of all loaded textures.
		/// </summary>
		/// <value>
		/// A list of all loaded textures.
		/// </value>
		static public TextureCollection Textures
		{
			get
			{
				return textures;
			}
		}
	}
}
