using SolerSoft.Utils;
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.Windows.Forms;

namespace TexturePicker
{
	/************************************************
	 * Enumerations
	 ***********************************************/
	/// <summary>
	/// Defines the locations where the ImageView border can be drawn.
	/// </summary>
	public enum ImageViewBorderLocation
	{
		/// <summary>
		/// The border is drawn around the control.
		/// </summary>
		Control,

		// The border is drawn around the image.
		Image
	};

	/// <summary>
	/// A control for displaying images centered and optionally in their correct aspect ratio.
	/// </summary>
	public class ImageView : System.Windows.Forms.Control
	{
		/************************************************
		 * Member Variables
		 ***********************************************/
		private bool aspectCorrect;						// Indicates if the image is to be displayed in its original aspect ratio.
		private Brush backBrush;						// Cached drawing brush for the backgroud.
		private ImageViewBorderLocation borderLocation;	// The location of the border.
		private BorderStyle borderStyle;				// The border style of the control.
		private Border3DStyle border3DStyle;			// The style of the border if BorderStyle is Fixed3D.
		private ButtonBorderStyle borderSingleStyle;	// The style of the border if BorderStyle is FixedSingle.
		private bool enlargeToFill;						// Indicates if the image will be enlarged to fill the view.
		private Image image;							// The full image set with the Image property.
		private InterpolationMode intermode;			// The interpolation mode used to scale the image.
		private Rectangle imageLoc;						// The location at which the image is to be drawn.
		private bool reduceToFit;						// Indicates if the image will be reduced to fit within the view.
		private double zoom;							// The zoom level the image is drawn at.

		/************************************************
		 * Constructors
		 ***********************************************/
		/// <summary>
		/// Creates a new <see cref="ImageView"/> instance.
		/// </summary>
		public ImageView()
		{
			// Set defaults
			aspectCorrect = true;
			borderStyle = BorderStyle.None;
			border3DStyle = Border3DStyle.Etched;
			borderSingleStyle = ButtonBorderStyle.Solid;
			intermode = InterpolationMode.HighQualityBilinear;	// Nice smooth scaling
			ResizeRedraw = true;
			TabStop = false;
			zoom = 1;

			// Update control styles for double-buffering
			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			SetStyle(ControlStyles.DoubleBuffer, true);
			SetStyle(ControlStyles.Opaque, false);
			SetStyle(ControlStyles.Selectable, false);
			SetStyle(ControlStyles.SupportsTransparentBackColor, true);
			SetStyle(ControlStyles.UserPaint, true);

			// Create graphics objects
			backBrush = new SolidBrush(BackColor);
		}

		/************************************************
		 * Internal Methods
		 ***********************************************/
		/// <summary>
		/// Updates the location where the image is to be drawn based on its dimensions and the 
		/// <see cref="AspectCorrect"/> option.
		/// </summary>
		private void UpdateLocation()
		{
			// If there is no image, ignore
			if (image == null) return;

			// If not aspect correct, just use client area
			if (!aspectCorrect)
			{
				imageLoc = ClientRectangle;
			}
			else
			{
				// Start with imgae size
				Size drawSize = image.Size;

				// Zoom?
				if (zoom != 1) Metrics.Scale(ref drawSize, zoom);

				// Create the draw location
				imageLoc = new Rectangle(Point.Empty, drawSize);

				// Center and scale over client area
				Metrics.CenterScale(ref imageLoc, ClientRectangle, enlargeToFill, reduceToFit);
			}
		}

		/************************************************
		 * Overrides / Event Handlers
		 ***********************************************/		
		/// <summary>
		/// Handles the painting of the control.
		/// </summary>
		/// <param name="e">
		/// A <see cref="PaintEventArgs"/> instance that contains the event data.
		/// </param>
		protected override void OnPaint(PaintEventArgs pe)
		{
			// Draw image if valid
			if (image != null)
			{
				// Set the interpolation mode
				pe.Graphics.InterpolationMode = intermode;

				// Draw the image into its draw location
				pe.Graphics.DrawImage(image, imageLoc);
			}
			else
			{
				// Just clear
				pe.Graphics.Clear(BackColor);
			}

			// Draw a border?
			if (borderStyle != BorderStyle.None)
			{
				// Placeholder
				Rectangle borderRect = Rectangle.Empty;

				// Determine location
				if ((borderLocation == ImageViewBorderLocation.Control) || (image == null))
				{
					borderRect = ClientRectangle;
				}
				else
				{
					borderRect = imageLoc;
				}

				// Which border to draw?
				switch (borderStyle)
				{
					case BorderStyle.Fixed3D:
						/*
						Rectangle bRect = borderRect;
						bRect.Width -= 4; // TODO: Shouldn't this be in the system somewhere?
						bRect.Height -= 4;
						*/
						ControlPaint.DrawBorder3D(pe.Graphics, borderRect, border3DStyle);
						break;
					case BorderStyle.FixedSingle:
						ControlPaint.DrawBorder(pe.Graphics, borderRect, ForeColor, borderSingleStyle);
						break;
				}
			}
		}

		protected override void OnBackColorChanged(EventArgs e)
		{
			// Update graphics
			backBrush = new SolidBrush(BackColor);

			// Redraw
			Invalidate();
		}

		protected override void OnForeColorChanged(EventArgs e)
		{
			// Redraw
			Invalidate();
		}

		/// <summary>
		/// Handles the size of the control changing.
		/// </summary>
		/// <param name="e">
		/// A <see cref="EventArgs"/> instance that contains the event data.
		/// </param>
		protected override void OnSizeChanged(EventArgs e)
		{
			// Update the drawing location
			UpdateLocation();

			// Pass to base to complete
			base.OnSizeChanged(e);
		}


		/************************************************
		 * Properties
		 ***********************************************/
		/// <summary>
		/// Gets or sets the location where the border is drawn.
		/// </summary>
		/// <value>
		/// An <see cref="ImageViewBorderLocation"/> that indicates where the border is drawn.
		/// </value>
		[DefaultValue(ImageViewBorderLocation.Control)]
		public ImageViewBorderLocation BorderLocation
		{
			get
			{
				return borderLocation;
			}
			set
			{
				// Ensure changing
				if (value == borderLocation) return;
  
				// Store
				borderLocation = value;

				// Redraw
				Invalidate();
			}
		}

		/// <summary>
		/// Gets or sets the border style for the control.
		/// </summary>
		/// <value>
		/// One of the <see cref="BorderStyle"/> enumeration values. The default is None.
		/// </value>
		[DefaultValue(BorderStyle.None)]
		public BorderStyle BorderStyle
		{
			get
			{
				return borderStyle;
			}
			set
			{
				// Ensure changing
				if (value == borderStyle) return;

				// Store
				borderStyle = value;

				// Redraw
				Invalidate();
			}
		}

		/// <summary>
		/// Gets or sets the style of the 3D border if <see cref="BorderStyle"/> is 
		/// <see cref="BorderStyle.Fixed3D">Fixed3D</see>.
		/// </summary>
		/// <value>
		/// One of the <see cref="Border3DStyle"/> enumeration values. The default is Etched.
		/// </value>
		[DefaultValue(Border3DStyle.Etched)]
		public Border3DStyle Border3DStyle
		{
			get
			{
				return border3DStyle;
			}
			set
			{
				// Ensure changing
				if (value == border3DStyle) return;
  
				// Store
				border3DStyle = value;

				// Redraw?
				if (borderStyle == BorderStyle.Fixed3D) Invalidate();
			}
		}

		/// <summary>
		/// Gets or sets the style of the single border if <see cref="BorderStyle"/> is 
		/// <see cref="BorderStyle.FixedSingle">FixedSingle</see>.
		/// </summary>
		/// <value>
		/// One of the <see cref="ButtonBorderStyle"/> enumeration values. The default is Solid.
		/// </value>
		[DefaultValue(ButtonBorderStyle.Solid)]
		public ButtonBorderStyle BorderSingleStyle
		{
			get
			{
				return borderSingleStyle;
			}
			set
			{
				// Ensure changing
				if (value == borderSingleStyle) return;
  
				// Store
				borderSingleStyle = value;

				// Redraw?
				if (borderStyle == BorderStyle.FixedSingle) Invalidate();
			}
		}

		/// <summary>
		/// Gets or sets a value that indicates if the image will be enlarged to fill the view.
		/// </summary>
		/// <value>
		/// true if the image should be enlarged to fill the view; otherwise false.
		/// </value>
		[DefaultValue(false)]
		public bool EnlargeToFill
		{
			get
			{
				return enlargeToFill;
			}
			set
			{
				// Ensure changing
				if (value == enlargeToFill) return;
  
				// Store
				enlargeToFill = value;

				// Calculate drawing location for new image
				UpdateLocation();

				// Redraw
				Invalidate();
			}
		}

		/// <summary>
		/// Gets or sets the <see cref="InterpolationMode"/> used to scale the image.
		/// </summary>
		[DefaultValue(InterpolationMode.HighQualityBilinear)]
		public InterpolationMode InterpolationMode
		{
			get
			{
				return intermode;
			}
			set
			{
				// Ensure changing
				if (value == intermode) return;

				// Store the new interpolation mode
				intermode = value;

				// If image is valid, redraw
				if (image != null) Invalidate();
			}
		}

		/// <summary>
		/// Gets or sets the <see cref="Image"/> that the <see cref="ImageView"/> displays.
		/// </summary>
		[DefaultValue(null)]
		public Image Image
		{
			get
			{
				return image;
			}
			set
			{
				// Ensure changing
				if (value == image) return;

				// Store
				image = value;

				// Calculate drawing location for new image
				UpdateLocation();

				// Redraw
				Invalidate();
			}
		}

		/// <summary>
		/// Gets or sets a value that indicates if the image will be reduced to fit within the view.
		/// </summary>
		/// <value>
		/// true if the image should be reduced to fit within the view; otherwise false.
		/// </value>
		[DefaultValue(false)]
		public bool ReduceToFit
		{
			get
			{
				return reduceToFit;
			}
			set
			{
				// Ensure changing
				if (value == reduceToFit) return;
  
				// Store
				reduceToFit = value;

				// Calculate drawing location for new image
				UpdateLocation();

				// Redraw
				Invalidate();
			}
		}

		/// <summary>
		/// Gets or sets the zoom level the image is drawn at.
		/// </summary>
		/// <value>
		/// The zoom level the image is drawn at.
		/// </value>
		/// <remarks>
		/// This value may not be honored if <see cref="EnlargeToFill"/> or <see cref="ReduceToFit"/> is enabled, 
		/// depending on the size of the control.
		/// </remarks>
		public double Zoom
		{
			get
			{
				return zoom;
			}
			set
			{
				// Ensure changing
				if (zoom == value) return;
  
				// Store
				zoom = value;

				// Redraw?
				if (image != null) 
				{
					// Calculate drawing location for new image
					UpdateLocation();

					// Redraw
					Invalidate();
				}
			}
		}
	}
}
