using System;
using System.Drawing;

namespace SolerSoft.Utils
{
	/************************************************
	 * Enumerations
	 ***********************************************/
	/// <summary>
	/// Specifies alignment of content on the drawing surface.
	/// </summary>
	public enum ContentAlignment
	{
		/// <summary>
		/// Content is vertically aligned at the bottom, and horizontally aligned at the center.
		/// </summary>
		BottomCenter	= 6,

		/// <summary>
		/// Content is vertically aligned at the bottom, and horizontally aligned on the left.
		/// </summary>
		BottomLeft		= 7,

		/// <summary>
		/// Content is vertically aligned at the bottom, and horizontally aligned on the right.
		/// </summary>
		BottomRight		= 8,

		/// <summary>
		/// Content is vertically aligned in the middle, and horizontally aligned at the center.
		/// </summary>
		MiddleCenter	= 0,

		/// <summary>
		/// Content is vertically aligned in the middle, and horizontally aligned on the left.
		/// </summary>
		MiddleLeft		= 1,

		/// <summary>
		/// Content is vertically aligned in the middle, and horizontally aligned on the right.
		/// </summary>
		MiddleRight		= 2,

		/// <summary>
		/// Content is vertically aligned at the top, and horizontally aligned at the center.
		/// </summary>
		TopCenter		= 3,

		/// <summary>
		/// Content is vertically aligned at the top, and horizontally aligned on the left.
		/// </summary>
		TopLeft			= 4,

		/// <summary>
		/// Content is vertically aligned at the top, and horizontally aligned on the right.
		/// </summary>
		TopRight		= 5
	}

	/// <summary>
	/// Utility methods for working with metrics.
	/// </summary>
	public class Metrics
	{
		/************************************************
		 * Methods
		 ***********************************************/
		/// <summary>
		/// Centers a rectangle over another.
		/// </summary>
		/// <param name="rect">
		/// The <see cref="Rectangle"/> to center.
		/// </param>
		/// <param name="over">
		/// The <see cref="Rectangle"/> to center over.
		/// </param>
		static public void Center(ref Rectangle rect, Rectangle over)
		{
			int OX = (over.Width - rect.Width) / 2;
			int OY = (over.Height - rect.Height) / 2;

			rect.Offset(OX,OY);
		}

		/// <summary>
		/// Centers a rectangle within another and scales the rectangle to fit if needed.
		/// </summary>
		/// <param name="source">
		/// The source <see cref="Rectangle"/> to center and optionally scale.
		/// </param>
		/// <param name="target">
		/// The target <see cref="Rectangle"/> to center on.
		/// </param>
		/// <param name="expand">
		/// true if the source should expanded to fill the target area.
		/// </param>
		/// <param name="contract">
		/// true if the source should contract to fit within the target area.
		/// </param>
		static public void CenterScale(ref Rectangle source, Rectangle target, bool expand, bool contract)
		{
			Scale(ref source, target, expand, contract);
			Center(ref source, target);
		}

		/// <summary>
		/// Scales the specified rectangle to fit within another while mainting the aspect ratio.
		/// </summary>
		/// <param name="source">
		/// The source <see cref="Rectangle"/> to center and optionally scale.
		/// </param>
		/// <param name="target">
		/// The target <see cref="Rectangle"/> to center on.
		/// </param>
		/// <param name="expand">
		/// true if the source should expanded to fill the target area.
		/// </param>
		/// <param name="contract">
		/// true if the source should contract to fit within the target area.
		/// </param>
		static public void Scale(ref Rectangle source, Rectangle target, bool expand, bool contract)
		{
			// What's the orientation?
			bool wide = source.Width > source.Height;

			// Can we expand?
			bool canExpand = false;
			if (wide)
			{
				canExpand = source.Width < target.Width;
			}
			else
			{
				canExpand = source.Height < target.Height;
			}

			// Can we contract?
			bool canContract = false;
			if (wide)
			{
				canContract = source.Width > target.Width;
			}
			else
			{
				canContract = source.Height > target.Height;
			}

			// Expand or contract?
			if ((canExpand && expand) || (canContract && contract))
			{
				// Determin scale
				float haspect = (float)target.Height / source.Height;
				float waspect = (float)target.Width / source.Width;

				// Scale based on orientation
				if (wide)
				{
					source.Width = target.Width;
					source.Height = (int)(waspect * source.Height);
				}
				else
				{
					source.Height = target.Height;
					source.Width = (int)(haspect * source.Width);
				}
			}
		}

		/// <summary>
		/// Scales the specified size structure to fit within another, mainting the sizes aspect ratio.
		/// </summary>
		/// <param name="size">
		/// The <see cref="Size"/> structure to scale.
		/// </param>
		/// <param name="within">
		/// The <see cref="Size"/> structure that the structure must fit within.
		/// </param>
		static public void Scale(ref Size size, Size within)
		{
			// If it already fits, just return
			if ((size.Width <= within.Width) && (size.Height <= within.Height)) return;

			// Determin scale
			float haspect = (float)within.Height / size.Height;
			float waspect = (float)within.Width / size.Width;

			// Scale based on orientation
			if (haspect < waspect)
			{
				size.Height = within.Height;
				size.Width = (int)(haspect * size.Width);
			}
			else
			{
				size.Width = within.Width;
				size.Height = (int)(waspect * size.Height);
			}
		}
	
		/// <summary>
		/// Scales the specified size structure to a percentage of its original value.
		/// </summary>
		/// <param name="size">
		/// The <see cref="Size"/> structure to scale.
		/// </param>
		/// <param name="scale">
		/// The percentage of the original value to scale to.
		/// </param>
		static public void Scale(ref Size size, double scale)
		{
			size = new Size((int)(size.Width * scale), (int)(size.Height * scale));
		}

		/// <summary>
		/// Rotates a <see cref="Size"/> structure; swapping the width and height.
		/// </summary>
		/// <param name="size">
		/// The <see cref="Size"/> structure to rotate.
		/// </param>
		static public void Rotate(ref Size size)
		{
			int H = size.Height;
			size.Height = size.Width;
			size.Width = H;
		}
	}
}
