using System;
using System.IO;
using System.Windows.Forms;

namespace TexturePicker
{
	/// <summary>
	/// Defines a source for a texture.
	/// </summary>
	public class Source
	{
		/************************************************
		 * Member Variables
		 ***********************************************/
		private TextureComponents components;
		private bool haveCombined;
		private FileInfo mainFile;
		private string name;
		private SourceFileCollection associatedFiles;

		/************************************************
		 * Constructors
		 ***********************************************/
		/// <summary>
		/// Creates a new <see cref="Source"/> instance.
		/// </summary>
		public Source(string name)
		{
			// Validate
			if (name == null) throw new ArgumentNullException("name");
			if (name == string.Empty) throw new ArgumentException("name");

			// Create vars
			associatedFiles = new SourceFileCollection(this);

			// Store
			this.name = name;
		}

		/************************************************
		 * Internal Methods
		 ***********************************************/
		/// <summary>
		/// Gets the type of texture information stored within the specified file.
		/// </summary>
		/// <param name="file">
		/// The file to check.
		/// </param>
		/// <returns>
		/// A <see cref="TextureType"/> value specifying the texture components stored within the file.
		/// </returns>
		static private TextureComponents GetTextureComponents(FileInfo file)
		{
			// Start with unknown
			TextureComponents comp = TextureComponents.Unknown;

			// Get the file name and convert it to lower case
			string fileName = file.Name.ToLower();
			
			// Look for components based on file name
			if (fileName.IndexOf("_all") > -1)
			{
				comp = TextureComponents.Combined;
			}
			else if (fileName.IndexOf("_rgb") > -1)
			{
				comp = TextureComponents.RGB;
			}
			else if (fileName.IndexOf("_a") > -1)
			{
				comp = TextureComponents.AlphaMask;
			}

			// Done
			return comp;
		}

		/// <summary>
		/// Allows the source to validate if a faile should be added to the collection.
		/// </summary>
		/// <param name="file">
		/// The file that is to be added.
		/// </param>
		/// <returns>
		/// true if the file is to be added; otherwise false.
		/// </returns>
		internal bool OnFileInsert(FileInfo file)
		{
			// If we already have a combined texture, no other files are allowed
			if (haveCombined) return false;

			// Get the components of this file
			TextureComponents comp = GetTextureComponents(file);

			// Work based on the components in the file
			if (comp == TextureComponents.Combined)
			{
				// We now have a combined texture
				haveCombined = true;

				// We now have all components
				components = comp;

				// Cleare out other files
				associatedFiles.Clear();

				// Select as main file
				mainFile = file;
			}
			else if (comp == TextureComponents.RGB)
			{
				// If we don't already have an RGB component, select this as the main file
				if ((components & TextureComponents.RGB) != TextureComponents.RGB) mainFile = file;

				// Update our components to include RGB
				components |= TextureComponents.RGB;
			}
			else if (comp == TextureComponents.AlphaMask)
			{
				// If we don't already have an RGB component, select this as the main file
				if ((components & TextureComponents.RGB) != TextureComponents.RGB) mainFile = file;

				// Update our components to include Alpha
				components |= TextureComponents.AlphaMask;
			}
			else
			{
				// This is an unknown texture component
				MessageBox.Show(string.Format("Unknown texture component '{0}', ignored.", file.FullName));

				// Do not add
				return false;
			}

			// Allow the add
			return true;
		}

		/// <summary>
		/// Notifies the source when a file is removed.
		/// </summary>
		/// <param name="file">
		/// The file that was removed from the associated files collection.
		/// </param>
		internal void OnFileRemoved(FileInfo file)
		{
			// Get the components of the file
			TextureComponents comp = GetTextureComponents(file);

			// Remove from the list of components
			components ^= comp;

			// If this file was combined, we need to uncheck it
			if (comp == TextureComponents.Combined) haveCombined = false;
		}

		/************************************************
		 * Properties
		 ***********************************************/
		/// <summary>
		/// Gets the texture components supplied by this source.
		/// </summary>
		/// <value>
		/// The texture components supplied by this source.
		/// </value>
		public TextureComponents Components
		{
			get
			{
				return components;
			}
		}

		/// <summary>
		/// Gets a collection of files associated with this source.
		/// </summary>
		/// <value>
		/// A collection of files associated with this source.
		/// </value>
		public SourceFileCollection AssociatedFiles
		{
			get
			{
				return associatedFiles;
			}
			set
			{
				associatedFiles = value;
			}
		}

		/// <summary>
		/// Gets the main file of the Source.
		/// </summary>
		/// <value>
		/// The main file of the Source.
		/// </value>
		public FileInfo MainFile
		{
			get
			{
				return mainFile;
			}
		}

		/// <summary>
		/// Gets or sets the name of the Source.
		/// </summary>
		/// <value>
		/// The name of the Source.
		/// </value>
		public string Name
		{
			get
			{
				return name;
			}
			set
			{
				// Validate
				if (value == null) throw new ArgumentNullException("value");
				if (value == string.Empty) throw new ArgumentException("value");

				// Store
				name = value;
			}
		}
	}
}
