using System;
using System.IO;
using System.Collections;

namespace TexturePicker
{
	/// <summary>
	/// A collection of <see cref="Source"/> objects.
	/// </summary>
	public class SourceCollection : CollectionBase
	{
		/************************************************
		 * Member Variables
		 ***********************************************/
		private Hashtable sourceByName;

		/************************************************
		 * Constructors
		 ***********************************************/
		/// <summary>
		/// Creates a new <see cref="SourceCollection"/> instance.
		/// </summary>
		public SourceCollection()
		{
			// Create vars
			sourceByName = new Hashtable();
		}

		/************************************************
		 * Overrides / Event Handlers
		 ***********************************************/
		protected override void OnClear()
		{
			// Pass to base first
			base.OnClear();

			// Clear hash
			sourceByName.Clear();
		}

		protected override void OnInsertComplete(int index, object value)
		{
			// Pass to base first
			base.OnInsertComplete (index, value);

			// Cast
			Source source = (Source)value;

			// Add to hash
			sourceByName[source.Name] = source;
		}

		protected override void OnRemoveComplete(int index, object value)
		{
			// Pass to base first
			base.OnRemoveComplete (index, value);

			// Cast
			Source source = (Source)value;

			// Remove from hash
			sourceByName.Remove(source.Name);
		}

		protected override void OnValidate(object value)
		{
			// Let base validate first
			base.OnValidate (value);

			// Validate object is Source
			if (!(value is Source)) throw new ArgumentException("Object must be of type Source.");
		}

		/************************************************
		 * Methods
		 ***********************************************/
		/// <summary>
		/// Adds the specified <see cref="Source"/> to the collection.
		/// </summary>
		/// <param name="value">
		/// The <see cref="Source"/> to add.
		/// </param>
		/// <returns>
		/// The index of the newly added <see cref="Source"/>.
		/// </returns>
		public int Add(Source value)
		{
			return List.Add(value);
		}
		
		/// <summary>
		/// Gets a value that indicates if the specified <see cref="Source"/> exists within the collection.
		/// </summary>
		/// <param name="value">
		/// The <see cref="Source"/> to check for.
		/// </param>
		/// <returns>
		/// true if the specified <see cref="Source"/> exists within the collection; otherwise false.
		/// </returns>
		public bool Contains(Source value)
		{
			return (IndexOf(value) > -1);
		}

		/// <summary>
		/// Gets the index of the specified <see cref="Source"/> within the collection.
		/// </summary>
		/// <param name="value">
		/// The <see cref="Source"/> to obtain the index of.
		/// </param>
		/// <returns>
		/// The index of the specified <see cref="Source"/> if found within the collection; otherwise -1;
		/// </returns>
		public int IndexOf(Source value)
		{
			return List.IndexOf(value);
		}

		/// <summary>
		/// Removes the specified <see cref="Source"/> from the collection.
		/// </summary>
		/// <param name="value">
		/// The <see cref="Source"/> to remove.
		/// </param>
		public void Remove(Source value)
		{
			List.Remove(value);
		}
		
		/************************************************
		 * Properties
		 ***********************************************/
		/// <summary>
		/// Gets or sets the <see cref="Source"/> at the specified index.
		/// </summary>
		public Source this[int index]
		{
			get
			{
				return (Source)List[index];
			}
			set
			{
				List[index] = value;
			}
		}

		/// <summary>
		/// Gets the source with the specified name in the collection.
		/// </summary>
		public Source this[string name]
		{
			get
			{
				return (Source)sourceByName[name];
			}
		}
	}
}
