using System;
using System.IO;
using System.Collections;

namespace TexturePicker
{
	/// <summary>
	/// A collection of <see cref="FileInfo"/> objects.
	/// </summary>
	public class SourceFileCollection : CollectionBase
	{
		/************************************************
		 * Member Variables
		 ***********************************************/
		private Source source;
		private object remove;

		/************************************************
		 * Constructors
		 ***********************************************/
		/// <summary>
		/// Creates a new <see cref="SourceFileCollection"/> instance.
		/// </summary>
		/// <param name="source">
		/// The parent <see cref="Source"/> that this collection belongs to.
		/// </param>
		public SourceFileCollection(Source source)
		{
			// Validate
			if (source == null) throw new ArgumentNullException("source");

			// Store
			this.source = source;
		}

		/// <summary>
		/// Gets a value that indicates if the file should be considered the mail file for a source.
		/// </summary>
		/// <param name="file">
		/// The file to consider for the main file.
		/// </param>
		/// <returns>
		/// true if the file should be the main file; otherwise false.
		/// </returns>
		static private bool IsMainFile(FileInfo file)
		{
			// Get the file name and convert it to lower case
			string fileName = file.Name.ToLower();
			
			// Look for 'rgb' in the file name
			int iAll = fileName.IndexOf("all");
			int iRgb = fileName.IndexOf("rgb");
			return ((iAll > -1) || (iRgb > -1));
		}

		/************************************************
		 * Overrides / Event Handlers
		 ***********************************************/
		protected override void OnInsert(int index, object value)
		{
			// Check with source if we can keep it
			if (!source.OnFileInsert((FileInfo)value))
			{
				remove = value;
			}

			base.OnInsert (index, value);
		}

		protected override void OnInsertComplete(int index, object value)
		{
			// Remove?
			if (remove == value) InnerList.RemoveAt(index);
		}


		protected override void OnRemoveComplete(int index, object value)
		{
			// Let base remove
			base.OnRemoveComplete (index, value);
			
			// Notify source
			source.OnFileRemoved((FileInfo)value);
		}

		
		protected override void OnValidate(object value)
		{
			// Let base validate first
			base.OnValidate (value);

			// Validate object is FileInfo
			if (!(value is FileInfo)) throw new ArgumentException("Object must be of type FileInfo.");
		}

		/************************************************
		 * Methods
		 ***********************************************/
		/// <summary>
		/// Adds the specified <see cref="FileInfo"/> to the collection.
		/// </summary>
		/// <param name="value">
		/// The <see cref="FileInfo"/> to add.
		/// </param>
		/// <returns>
		/// The index of the newly added <see cref="FileInfo"/>.
		/// </returns>
		public int Add(FileInfo value)
		{
			return List.Add(value);
		}
		
		/// <summary>
		/// Gets a value that indicates if the specified <see cref="FileInfo"/> exists within the collection.
		/// </summary>
		/// <param name="value">
		/// The <see cref="FileInfo"/> to check for.
		/// </param>
		/// <returns>
		/// true if the specified <see cref="FileInfo"/> exists within the collection; otherwise false.
		/// </returns>
		public bool Contains(FileInfo value)
		{
			return (IndexOf(value) > -1);
		}

		/// <summary>
		/// Gets the index of the specified <see cref="FileInfo"/> within the collection.
		/// </summary>
		/// <param name="value">
		/// The <see cref="FileInfo"/> to obtain the index of.
		/// </param>
		/// <returns>
		/// The index of the specified <see cref="FileInfo"/> if found within the collection; otherwise -1;
		/// </returns>
		public int IndexOf(FileInfo value)
		{
			return List.IndexOf(value);
		}

		/// <summary>
		/// Removes the specified <see cref="FileInfo"/> from the collection.
		/// </summary>
		/// <param name="value">
		/// The <see cref="FileInfo"/> to remove.
		/// </param>
		public void Remove(FileInfo value)
		{
			List.Remove(value);
		}
		
		/************************************************
		 * Properties
		 ***********************************************/
		/// <summary>
		/// Gets or sets the <see cref="FileInfo"/> at the specified index.
		/// </summary>
		public FileInfo this[int index]
		{
			get
			{
				return (FileInfo)List[index];
			}
			set
			{
				List[index] = value;
			}
		}
	}
}
