using System;

namespace TexturePicker
{
	/************************************************
	 * Enumerations
	 ***********************************************/
	/// <summary>
	/// Represents the type of texture information stored in an image file.
	/// </summary>
	[Flags]
	public enum TextureComponents
	{
		/// <summary>
		/// Texture type is unknown
		/// </summary>
		Unknown = 0,

		/// <summary>
		/// Image contains an Alpha mask.
		/// </summary>
		AlphaMask = 1,

		/// <summary>
		/// Image contains RGB data.
		/// </summary>
		RGB = 2,

		/// <summary>
		/// Image contains an Alpha mask and RGB data.
		/// </summary>
		Combined = 3
	};

	/// <summary>
	/// Represents a texture with one or more sources.
	/// </summary>
	public class Texture
	{
		/************************************************
		 * Member Variables
		 ***********************************************/
		private string name;
		private Source selectedSource;
		private SourceCollection sources;

		/************************************************
		 * Constructors
		 ***********************************************/
		/// <summary>
		/// Creates a new <see cref="Texture"/> instance.
		/// </summary>
		public Texture(string name)
		{
			// Validate
			if (name == null) throw new ArgumentNullException("name");
			if (name == string.Empty) throw new ArgumentException("name");

			// Create vars
			sources = new SourceCollection();

			// Store
			this.name = name;
		}

		/************************************************
		 * Properties
		 ***********************************************/
		/// <summary>
		/// Gets or sets the name of the Texture.
		/// </summary>
		/// <value>
		/// The name of the Texture.
		/// </value>
		public string Name
		{
			get
			{
				return name;
			}
			set
			{
				// Validate
				if (value == null) throw new ArgumentNullException("value");
				if (value == string.Empty) throw new ArgumentException("value");

				// Store
				name = value;
			}
		}

		/// <summary>
		/// Gets or sets the selected source for the Texture.
		/// </summary>
		/// <value>
		/// The selected source for the Texture.
		/// </value>
		/// <remarks>
		/// The selected source must exist within the <see cref="Sources"/> collection.
		/// </remarks>
		public Source SelectedSource
		{
			get
			{
				return selectedSource;
			}
			set
			{
				// Ensure changing
				if (selectedSource == value) return;
  
				// Ensure in sources
				if (!sources.Contains(value)) throw new ArgumentException("Specified source does not exist within sources collection.");

				// Store
				selectedSource = value;
			}
		}

		/// <summary>
		/// Gets a list of all sources for the Texture.
		/// </summary>
		/// <value>
		/// A list of all sources for the Texture.
		/// </value>
		public SourceCollection Sources
		{
			get
			{
				return sources;
			}
		}
	}
}
