using System;
using System.IO;
using System.Collections;

namespace TexturePicker
{
	/// <summary>
	/// A collection of <see cref="Texture"/> objects.
	/// </summary>
	public class TextureCollection : CollectionBase
	{
		/************************************************
		 * Member Variables
		 ***********************************************/
		private Hashtable textureByName;

		/************************************************
		 * Constructors
		 ***********************************************/
		/// <summary>
		/// Creates a new <see cref="TextureCollection"/> instance.
		/// </summary>
		public TextureCollection()
		{
			// Create vars
			textureByName = new Hashtable();
		}

		/************************************************
		 * Overrides / Event Handlers
		 ***********************************************/
		protected override void OnClear()
		{
			// Pass to base first
			base.OnClear();

			// Clear hash
			textureByName.Clear();
		}

		protected override void OnInsertComplete(int index, object value)
		{
			// Pass to base first
			base.OnInsertComplete (index, value);

			// Cast
			Texture texture = (Texture)value;

			// Add to hash
			textureByName[texture.Name] = texture;
		}

		protected override void OnRemoveComplete(int index, object value)
		{
			// Pass to base first
			base.OnRemoveComplete (index, value);

			// Cast
			Texture texture = (Texture)value;

			// Remove from hash
			textureByName.Remove(texture.Name);
		}

		protected override void OnValidate(object value)
		{
			// Let base validate first
			base.OnValidate (value);

			// Validate object is Texture
			if (!(value is Texture)) throw new ArgumentException("Object must be of type Texture.");
		}

		/************************************************
		 * Methods
		 ***********************************************/
		/// <summary>
		/// Adds the specified <see cref="Texture"/> to the collection.
		/// </summary>
		/// <param name="value">
		/// The <see cref="Texture"/> to add.
		/// </param>
		/// <returns>
		/// The index of the newly added <see cref="Texture"/>.
		/// </returns>
		public int Add(Texture value)
		{
			return List.Add(value);
		}
		
		/// <summary>
		/// Gets a value that indicates if the specified <see cref="Texture"/> exists within the collection.
		/// </summary>
		/// <param name="value">
		/// The <see cref="Texture"/> to check for.
		/// </param>
		/// <returns>
		/// true if the specified <see cref="Texture"/> exists within the collection; otherwise false.
		/// </returns>
		public bool Contains(Texture value)
		{
			return (IndexOf(value) > -1);
		}

		/// <summary>
		/// Gets the index of the specified <see cref="Texture"/> within the collection.
		/// </summary>
		/// <param name="value">
		/// The <see cref="Texture"/> to obtain the index of.
		/// </param>
		/// <returns>
		/// The index of the specified <see cref="Texture"/> if found within the collection; otherwise -1;
		/// </returns>
		public int IndexOf(Texture value)
		{
			return List.IndexOf(value);
		}

		/// <summary>
		/// Removes the specified <see cref="Texture"/> from the collection.
		/// </summary>
		/// <param name="value">
		/// The <see cref="Texture"/> to remove.
		/// </param>
		public void Remove(Texture value)
		{
			List.Remove(value);
		}
		
		/************************************************
		 * Properties
		 ***********************************************/
		/// <summary>
		/// Gets or sets the <see cref="Texture"/> at the specified index.
		/// </summary>
		public Texture this[int index]
		{
			get
			{
				return (Texture)List[index];
			}
			set
			{
				List[index] = value;
			}
		}

		/// <summary>
		/// Gets the texture with the specified name in the collection.
		/// </summary>
		public Texture this[string name]
		{
			get
			{
				return (Texture)textureByName[name];
			}
		}
	}
}
